/***************************************************************************
 *   Copyright (C) 2004 by Zhang Wei                                       *
 *   brep@163.com                                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <stdlib.h>
#include <iostream>
#include "charimage.hxx"

using std::cerr;
using std::endl;

CharImage::CharImage(int widthInBits, int height)
{
	charWidthInBits = widthInBits;
	charWidthInBytes = charWidthInBits / 8;
	if ((charWidthInBits % 8) != 0)
		charWidthInBytes++;
	charHeight = height;
	expandBits = charWidthInBytes * 8 - charWidthInBits;

	int len = charWidthInBytes * charHeight;
	for (int i=0; i<len; i++)
		image.push_back('\0');
}

bool CharImage::testBit(int x, int y)
{
	if (( x >= charWidthInBits) ||
	    ( y >= charHeight )) {
		cerr << "CharImage::testBit out of range!" << endl;
		exit(1);
	}
	
	//x += expandBits;

	int pos = y * charWidthInBytes + x / 8;

	bool result = image[pos] & (1 << (7 - (x % 8)));

	return result;
}

void CharImage::setBit(int x, int y)
{
	if (( x >= charWidthInBits ) ||
	    ( y >= charHeight )) {
		cerr << "CharImage::setBit out of range!" << endl;
		exit(1);
	}
	
	//x += expandBits;
	
	int pos = y * charWidthInBytes + x / 8;
	
	image[pos] |= (1 << (7 - (x % 8)));
}

void CharImage::clearBit(int x, int y)
{
	if (( x >= charWidthInBits ) ||
	    ( y >= charHeight )) {
		cerr << "CharImage::clearBit out of range!" << endl;
		exit(1);
	}
	
	//x += expandBits;
	
	int pos = y * charWidthInBytes + x / 8;
	
	image[pos] &= ~(1 << (7 - (x % 8)));
}
