/***************************************************************************
 *   Copyright (C) 2004 by Zhang Wei                                       *
 *   brep@163.com                                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <qpainter.h>
#include "fonteditor.hxx"

FontEditor::FontEditor(QWidget *parent, const char *name)
	:QWidget(parent, name, WStaticContents)
{
	setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);
	zoom = 8;
}

QSize FontEditor::sizeHint() const
{
	QSize size = zoom * QSize(charImage.width(),
				  charImage.height());
	if(zoom >= 3)
		size += QSize(1, 1);
	return size;
}


void FontEditor::setImage(const CharImage newImage)
{
	charImage = newImage;
	update();
	updateGeometry();
}

void FontEditor::setZoomFactor(int newZoom)
{
	if (newZoom < 1)
		newZoom = 1;

	if (newZoom != zoom) {
		zoom = newZoom;
		update();
		updateGeometry();
	}
}

void FontEditor::paintEvent(QPaintEvent *)
{
	QPainter painter(this);

	if(zoom >= 3) {
		painter.setPen(colorGroup().foreground());
		for (int i=0; i<=charImage.width(); ++i)
			painter.drawLine(zoom * i, 0,
					 zoom * i, zoom * charImage.height());
		for (int j=0; j<=charImage.height(); ++j)
			painter.drawLine(0, zoom *j,
					 zoom * charImage.width(), zoom * j);
	}

	for (int i=0; i<charImage.width(); ++i) {
		for (int j=0; j<charImage.height(); ++j)
			drawImagePixel(&painter, i, j);
	}
}

void FontEditor::drawImagePixel(QPainter *painter, int i, int j)
{
	QColor color;
	if(charImage.testBit(i, j))
		color = black;
	else
		color = white;
	
	if (zoom >= 3) {
		painter->fillRect(zoom * i + 1, zoom * j + 1,
				  zoom - 1, zoom -1, color);
	}
	else {
		painter->fillRect(zoom * i, zoom * j,
				  zoom, zoom, color);
	}
}

void FontEditor::mousePressEvent(QMouseEvent *event)
{
	if (event->button() == LeftButton)
		setImagePixel(event->pos(), true);
	else if (event->button() == RightButton)
		setImagePixel(event->pos(), false);
}

void FontEditor::mouseMoveEvent(QMouseEvent *event)
{
	if (event->state() & LeftButton)
		setImagePixel(event->pos(), true);
	else if (event->state() & RightButton)
		setImagePixel(event->pos(), false);
}

void FontEditor::setImagePixel(const QPoint &pos, bool draw)
{
	int i = pos.x() / zoom;
	int j = pos.y() / zoom;

	QRect imageRect(0, 0, charImage.width(), charImage.height());

	if(imageRect.contains(i, j)) {
		if(draw)
			charImage.setBit(i, j);
		else
			charImage.clearBit(i, j);

		QPainter painter(this);
		drawImagePixel(&painter, i, j);
	}
}

	
