/***************************************************************************
 *   Copyright (C) 2004 by Zhang Wei                                       *
 *   brep@163.com                                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <qmessagebox.h>
#include <qbitmap.h>

#include "fonteditor.hxx"
#include "fontsheet.hxx"
#include "editdialog.hxx"



FontSheet::FontSheet(QWidget *parent, const char *name)
	:QTable(parent, name)
{
	numCols = 16;
	numRows = 2000;
	setSelectionMode(NoSelection);
	setReadOnly(true);
	clear();
	connect(this, SIGNAL(doubleClicked( int, int, int,const QPoint&)),
		this, SLOT(editImage(int, int, int, const QPoint&)));
	
}

void FontSheet::clear()
{
	setNumRows(0);
	setNumCols(0);
	setNumRows(numRows);
	setNumCols(numCols);
	setCurrentCell(0, 0);
 	for(int i=0; i<numCols; i++) {
 		setColumnWidth(i, 20);
	}
 	for(int j=0; j<numRows; j++) {
 		setRowHeight(j, 20);
 	}
	
	fontData.clear();
}


void FontSheet::editImage(int row, int col,
			  int button, const QPoint & mousePos)
{
	uint pos = row * 16 + col;
	if(pos >= fontData.size())
		return;
	CharImage curImage = fontData[pos];
	
	EditDialog dialog(this);
	dialog.setImage(curImage);
	
	if(dialog.exec()) {
		curImage = fontData[pos] = dialog.image();
 		QBitmap bm( curImage.expandedWidth(), curImage.height(), curImage, FALSE);
		setPixmap(row, col, bm);
		emit modified();
	}
	

}


bool FontSheet::readFile(const QString &fileName)
{

	clear();

	int file;
	file = open(fileName, O_RDONLY);

	// check if the file begin with "0x36 0x04"
	// a bpsf font file always begin with "0x36 0x04"
	unsigned char  magic[2];
	read(file, magic, 2);
 	if (magic[0] != 0x36 || magic[1] != 0x04) {
 		QMessageBox::information( this, tr("BPSF Editor"),
					  tr("This file is not a BPSF font file!"));
		return false;
 	}

	read(file, &filemode, 1);

	if (filemode == 4 || filemode == 5) {
		read(file, &imageHeight, 1);
		read(file, &imageWidth, 1);
		read(file, fontsize, 4);
	}
	else {
		imageWidth = 8;
		read(file, &imageHeight, 1);
	}


	CharImage aImage((int)imageWidth,(int)imageHeight);
	
	int len = aImage.size(); // number of bytes per character	
 	while( read(file, aImage, len) == len ) {
 		fontData.push_back(aImage);
 	}

 	int numRows = fontData.size() / numCols;
 	if ((fontData.size() % numCols) != 0)
 		numRows++;

 	setNumRows(0);
 	setNumCols(0);
	setNumRows(numRows);
 	setNumCols(numCols);

	int colWidth = aImage.expandedWidth() + 3;
 	for(int i=0; i<numCols; i++) {
 		setColumnWidth(i, colWidth );
	}
 	for(int j=0; j<numRows; j++) {
 		setRowHeight(j, int(imageHeight) + 3);
 	}

	int row = 0;
 	int col = 0;
 	vector<CharImage>::iterator iter;
 	for(iter=fontData.begin(); iter != fontData.end(); ++iter) {
 		aImage = *iter;
 		QBitmap bm( aImage.expandedWidth(), aImage.height(), aImage, FALSE);
 		setPixmap(row, col, bm);
 		if(++col == 16) {
 			col = 0;
 			row++;
 		}
 	}

	 if(::close(file) == -1) 
		return false;
	
	return true;

	
}

	
	
bool FontSheet::writeFile(const QString &fileName)
{
	int file;
	file = creat(fileName, 0644);

	// write the magic number "0x36 0x04"
	// a bpsf font file always begin with "0x36 0x04"
	unsigned char  magic[2];
	magic[0] = 0x36;
	magic[1] = 0x04;
	write(file, magic, 2);
 
	write(file, &filemode, 1);

	if (filemode == 4 || filemode == 5) {
		write(file, &imageHeight, 1);
		write(file, &imageWidth, 1);
		write(file, fontsize, 4);
	}
	else {
		imageWidth = 8;
		write(file, &imageHeight, 1);
	}

	int len = fontData[0].size();

	vector<CharImage>::iterator iter;
 	for(iter=fontData.begin(); iter != fontData.end(); ++iter) {
 		write(file, *iter, len);
	}
	
	if(::close(file) == -1) 
		return false;
	
	return true;

}


